%macro TEXTWRAP( FILEIN     /* name of input file containing text                    */
               , FILEOUT    /* name of output file to contain reformatted text       */
               , LINELEN=80 /* [optional] maximum length of line of reformatted text */
               , RECFM=v    /* [optional] output file record format                  */
               ) ;

    /* PURPOSE: read text as stream from &FILEIN, reformat to max of &LINELEN characters
     *          write to &FILEOUT
     *
     * EXAMPLE OF USE:
     *      %let FILE_IN  = c:\documents and settings\rbettin\my SAS files\9.1\flowtext\text_in ;
     *      %let FILE_OUT = c:\documents and settings\rbettin\my SAS files\9.1\flowtext\text_out ;
     *
     *      %TEXTWRAP( &FILEIN, &FILEOUT, linelen=100 )
     */

    data _null_ ;
        length  textin $ 32767 textout $ &LINELEN ;
        file "&FILEOUT" lrecl=&LINELEN recfm=&RECFM ;

        infile "&FILEIN" length=reclen lrecl=32767 ;
        input textin $varying32767. reclen ;

        if lengthn( textin ) <= &LINELEN
        then put textin ;
        else
        do while( lengthn( textin ) > &LINELEN ) ;
            textout = reverse( substr( textin, 1, &LINELEN )) ;

            ndx = index( textout, ' ' ) ;

            if ndx
            then do ;
                textout = reverse( substr( textout, ndx + 1 )) ;

                put textout ;

                textin = substr( textin, &LINELEN - ndx + 1 ) ;

            end ;

            if lengthn( textin ) <= &LINELEN then put textin ;
        end ;
    run ;

%mend TEXTWRAP ;

%let IN = Y:\My SAS Files\9.1\Test\Textwrap\longwords.txt ;
%let OUT = Y:\My SAS Files\9.1\Test\Textwrap\formatted.txt ;

%TEXTWRAP( &IN, &OUT, linelen=80 )
